@@ -58,6 +58,12 @@ class TypeAnnotation:
             return TypeAnnotation(self.args[0])
         return None
 
+    def get_underlying_if_new_type(self) -> Optional["TypeAnnotation"]:
+        # Using the same heuristic as pydantic: https://github.com/pydantic/pydantic/pull/223/files
+        if hasattr(self.raw_type, "__name__") and hasattr(self.raw_type, "__supertype__"):
+            return TypeAnnotation(getattr(self.raw_type, "__supertype__"))
+        return None
+
     def get_underlyings_if_union(self) -> List["TypeAnnotation"]:
         if self.origin is Union:
             return [TypeAnnotation(t) for t in self.args]
@@ -106,6 +112,21 @@ class TypeAnnotation:
             else:
                 return underlying_if_optional.validate(value)
 
+        # Handle unions
+        underlyings_if_union = self.get_underlyings_if_union()
+        if len(underlyings_if_union) > 0:
+            errors: List[str] = []
+            for underlying in underlyings_if_union:
+                new_value, error = underlying.validate(value)
+                if error is None:
+                    return new_value, None
+                else:
+                    errors.append(error)
+            return (
+                value,
+                f"value {value} did not match any type of union:\n - " + "\n - ".join(errors),
+            )
+
         # Handle lists
         underlying_if_list = self.get_underlying_if_list()
         if underlying_if_list is not None:
@@ -151,8 +172,10 @@ class TypeAnnotation:
                 f"{allowed_values_if_enum}",
             )
 
-        # TODO handle:
-        # - Union
+        # Handle new type
+        underlying_if_new_type = self.get_underlying_if_new_type()
+        if underlying_if_new_type is not None:
+            return underlying_if_new_type.validate(value)
 
         # We have to assert self.raw_type is a true `type`
         if not isinstance(self.raw_type, type):
